
from dataclasses import dataclass, InitVar
from datetime import datetime
import re
import time


@dataclass
class AccountLineDetails:
    hash: str

    @classmethod
    def from_dict(cls, **kwargs):
        args = {k: kwargs.get(k) for k in cls.__annotations__.keys()}
        return cls(**args)


@dataclass
class AccountDetails:
    return_code: int
    return_text: str
    return_data: str

    def __post_init__(self):
        self._confirmed_lines = \
            [
                AccountLineDetails.from_dict(**i)
                for i in self.return_data['confirmedLines']
            ]

    @classmethod
    def from_dict(cls, **kwargs):
        args = {k: kwargs.get(k) for k in cls.__annotations__.keys()}
        return cls(**args)

    @property
    def confirmed_lines(self):
        return self._confirmed_lines


def string2date(redPocketDateString):
    return datetime.strptime(redPocketDateString, '%m/%d/%Y').date()


@dataclass
class LineDetails:
    return_code: InitVar[int]
    return_text: InitVar[str]
    return_data: InitVar[str]
    title: InitVar[str]
    phone_number: str = '0000000000'
    voice_balance: int = 0
    messaging_balance: int = 0
    data_balance: int = 0
    timestamp: int = 0
    start_date: datetime.date = None
    end_date: datetime.date = None
    hash: str = ''

    def __post_init__(self, return_code, return_text, return_data, title):
        self.timestamp = time.time()
        self.phone_number = return_data['mdn']

        self.voice_balance = self._cleanse_data(return_data['voice_balance'])
        self.messaging_balance = \
            self._cleanse_data(return_data['messaging_balance'])
        self.data_balance = self._cleanse_data(return_data['data_balance'])

        self.start_date = string2date(return_data['recurring']['last_date'])
        self.end_date = string2date(return_data['aed'])

        mat = re.findall(r'id (\w+)', title)
        try:
            self.hash = mat[0]
        except IndexError:
            raise Warning(f"Unable to parse 'hash' from title ('{title}')")

    @classmethod
    def from_dict(cls, **kwargs):
        args = {k: kwargs.get(k) for k in cls.__annotations__.keys()}
        return cls(**args)

    def _cleanse_data(self, value):
        if value in ['Unlimited', 'N/A']:
            return -1
        else:
            return int(value.replace(',', ''))

    @property
    def days_remaining(self):
        return (self.end_date - datetime.now().date()).days




import re

import requests

from pyredpocket.details import AccountDetails, LineDetails

BASE_URL = 'https://www.redpocket.com/'
URL_LOGIN = BASE_URL + 'login'
URL_POST_LOGIN = BASE_URL + 'my-lines'
URL_GET_OTHER_LINES = BASE_URL + 'account/get-other-lines'
URL_GET_DETAILS = BASE_URL + 'account/get-details?id={}&type=api'


class RedPocketLoginError(Exception):
    pass


class RedPocket(object):

    def __init__(self, username, password, auto=True, hashes=[]):
        self._client = requests.Session()
        self._logged_in = False
        self._username = username
        self._password = password

        if not isinstance(hashes, list):
            hashes = [hashes]
        self.hashes = hashes
        self.details = []

        if auto:
            self.process()

    def _extract_csrf(self, text):
        return re.findall(r'name="csrf" value="([\w|-]+)"', text)[0]

    def process(self):
        if not self._logged_in:
            self._login()

        if self.hashes == []:
            self.hashes = self._get_hashes()

        for has in self.hashes:
            self.details.append(self._get_details(has))

    def _login(self):

        #
        req = self._client.get(URL_LOGIN)
        req.raise_for_status()
        data = {
            'mdn': self._username,
            'password': self._password,
            'csrf': self._extract_csrf(req.text),
        }

        req = self._client.post(URL_LOGIN, data=data)
        req.raise_for_status()

        if req.url == URL_POST_LOGIN:
            self._loggedIn = True
        else:
            raise RedPocketLoginError

    def _get_hashes(self):
        if not self._logged_in:
            self._login()

        req = self._client.get(URL_GET_OTHER_LINES)
        req.raise_for_status()

        processed_details = AccountDetails.from_dict(**req.json())
        return [i.hash for i in processed_details.confirmed_lines]

    def _get_details(self, hashId):
        req = self._client.get(URL_GET_DETAILS.format(hashId))
        req.raise_for_status()

        return LineDetails.from_dict(**req.json())
